<?php
// $Id: fivestar_comment.inc,v 1.1.2.10 2008/02/23 16:44:35 quicksketch Exp $

/**
 * Form callback. Returns the configuration form.
 */
function fivestar_color_form() {
  $form = array(
    '#submit' => array('fivestar_color_form_submit'),
    '#tree' => FALSE,
    '#theme' => 'fivestar_color_form',
    '#type' => 'fieldset',
    '#title' => t('Color scheme'),
    '#weight' => -1,
    '#attributes' => array('id' => 'fivestar_color_scheme_form'),
    '#description' => t('If you\'ve selected a <em>Custom color widget</em> above, you can futher customize your widget by choosing colors below.'),
  );

  $theme = variable_get('theme_default', 'garland');
  $theme_info = color_get_info($theme);
  $themes = list_themes();
  $theme_name = $themes[$theme]->name;

  // Create the list of available schemes.
  $fivestar_schemes = array(
    '#ffeb38,#fff385,#ffd60a,#ffe561,#b3b3b3,#e6e6e6' => t('Yellow'),
    '#ff383a,#ff5f61,#ef9a9c,#f3b3b4,#b3b3b3,#e6e6e6' => t('Red'),
    '#5b6df1,#7c8af4,#1022c1,#2e41ee,#b3b3b3,#e6e6e6' => t('Blue'),
    '#5cf15b,#7cf47c,#12c610,#30ee2e,#b3b3b3,#e6e6e6' => t('Green'),
  );
  $theme_schemes = array();
  if (!empty($theme_info['schemes'])) {
    foreach ($theme_info['schemes'] as $key => $name) {
      $colors = explode(',', $key);
      $theme_scheme = array(
        'on1' => $colors[0],
        'on2' => _fivestar_color_blend($colors[0], '#FFFFFF', .8),
        'hover1' => $colors[2],
        'hover2' => _fivestar_color_blend($colors[2], '#FFFFFF', .8),
        'off1' => $colors[4],
        'off2' => _fivestar_color_blend($colors[4], '#FFFFFF', .8),
      );
      $theme_schemes[implode(',', $theme_scheme)] = $name;
    }
  }

  if (!empty($theme_schemes)) {
    $options = array(
      t('Fivestar schemes') => $fivestar_schemes,
      t('@theme_name schemes', array('@theme_name' => drupal_ucfirst($theme_name))) => $theme_schemes,
    );
  }
  else {
    $options = $fivestar_schemes;
  }
  $options[''] = t('Custom');

  // See if we're using a predefined scheme.
  $current = implode(',', variable_get('fivestar_color_palette', explode(',', array_search(t('Yellow'), $fivestar_schemes))));

  // Add scheme selector.
  $form['scheme'] = array(
    '#type' => 'select',
    '#title' => t('Color set'),
    '#options' => $options,
    '#default_value' => $current,
  );

  // Add palette fields.
  $names = array(
    'on1' => t('On 1 color'),
    'on2' => t('On 2 color'),
    'hover1' => t('Hover 1 color'),
    'hover2' => t('Hover 2 color'),
    'off1' => t('Off 1 color'),
    'off2' => t('Off 2 color'),
  );

  $default_colors = explode(',', $current);
  $color_scheme = variable_get('fivestar_colors', $default_colors);

  $form['palette']['#tree'] = true;
  $form['palette']['#theme'] = 'fivestar_color_palette';
  $n = 0;
  foreach ($names as $key => $name) {
    $form['palette'][$key] = array(
      '#type' => 'textfield',
      '#title' => $name,
      '#default_value' => $color_scheme[$n],
      '#size' => 8,
    );
    $n++;
  }

  return $form;
}

/**
 * Theme color form.
 */
function theme_fivestar_color_form($form) {
  // Add Farbtastic color picker.
  drupal_add_css(drupal_get_path('module', 'color') .'/fivestar-admin.css', 'module', 'all', FALSE);
  drupal_add_css('misc/farbtastic/farbtastic.css', 'module', 'all', FALSE);
  drupal_add_js(drupal_get_path('module', 'fivestar') .'/fivestar-color.js');
  drupal_add_js('misc/farbtastic/farbtastic.js');

  // Add custom CSS/JS.
  $default_colors = array();
  foreach (element_children($form['palette']) as $key) {
    $default_colors[$key] = $form['palette'][$key]['#value'];
  }
  $color_scheme = variable_get('fivestar_colors', $default_colors);
  drupal_add_js(array('fivestar' => array('reference' => $color_scheme, 'colorPreview' => url('fivestar/preview/color'))), 'setting');

  // Wrapper.
  $output .= '<div class="color-form clear-block">';

  // Color schemes.
  $output .= drupal_render($form['scheme']);

  // Palette.
  $output .= '<div id="palette" class="clear-block">';
  foreach (element_children($form['palette']) as $name) {
    $output .= drupal_render($form['palette'][$name]);
  }
  $output .= '</div>';

  // Render the form.
  $output .= drupal_render($form);
  // Close wrapper.
  $output .= '</div>';

  return $output;
}

/**
 * Submit handler for color change form.
 */
function fivstar_color_form_submit($form_id, $values) {

  // Render new images
  //_color_render_images($star_set, $palette);

  // Rewrite stylesheet
  //_color_rewrite_stylesheet($star_set, $palette);

}

/**
 * Rewrite the stylesheet to match the colors in the palette.
 */
function _fivestar_color_rewrite_stylesheet($theme, $palette) {
  /*
  // Load stylesheet
  $style = file_get_contents($paths['source'] .'style.css');

  // Write new stylesheet
  $file = fopen($paths['stylesheet'], 'w+');
  fwrite($file, $output);
  fclose($file);
  $paths['files'][] = $paths['stylesheet'];

  // Set standard file permissions for webserver-generated files
  @chmod($paths['stylesheet'], 0664);
  */
}

/**
 * Render images that match a given palette.
 * @param $source
 *   The original image source. A matching mask image must be at the same
 *   location.
 * @param $palette
 *   The colors to be used in the generation of this image.
 *     0: on1
 *     1: on2
 *     2: hover1
 *     3: hover2
 *     4: off1
 *     5: off2
 */
function _fivestar_color_render($source, $palette) {

  // Prepare template image.
  $overlay = imagecreatefrompng($source);
  imagealphablending($overlay, true);
  imagesavealpha($overlay, true);

  $mask = imagecreatefrompng(str_replace('.png', '-mask.png', $source));
  imagealphablending($mask, true);
  imagesavealpha($mask, true);

  $width = imagesx($overlay);
  $height = imagesy($overlay);

  // Apply color mask.
  $slices = (basename($source) == 'star.png') ? 3 : 2;
  $slice_height = floor($height / $slices);

  for ($n = 0; $n < $slices; $n++) {
    $slice_y = $slice_height * $n;
    _fivestar_color_mask($mask, $palette[$n * 2], 0, $slice_y, $width, $slice_height);
  }

  // Apply the overlay on top of the mask.
  imagecopy($mask, $overlay, 0, 0, 0, 0, $width, $height);

  return $mask;
}

/**
 * Apply a color to a portion of a black and white mask image.
 * 
 * @param $mask
 *   A GD image reference containing the mask image.
 * @param $color
 *   A hex color value (i.e. ffdd00) to apply to the mask.
 */
function _fivestar_color_mask(&$mask, $color, $x_offset, $y_offset, $width, $height) {
  $rgb = _fivestar_color_unpack($color);
  for ($x = $x_offset; $x < $x_offset + $width; $x++) {
    for ($y = $y_offset; $y < $y_offset + $height; $y++) {
      $current_pixel = imagecolorsforindex($mask, imagecolorat($mask, $x, $y));
      $new_color = imagecolorallocatealpha($mask, $rgb[0], $rgb[1], $rgb[2], $current_pixel['alpha']);
      imagesetpixel($mask, $x, $y, $new_color);
    }
  }
}

/**
 * Shift a given color, using a reference pair and a target blend color.
 *
 * Note: this function is significantly different from the JS version, as it
 * is written to match the blended images perfectly.
 *
 * Constraint: if (ref2 == target + (ref1 - target) * delta) for some fraction delta
 *              then (return == target + (given - target) * delta)
 *
 * Loose constraint: Preserve relative positions in saturation and luminance
 *                   space.
 * @param $given
 *   Hex color value.
 * @param $ref1
 *   Hex color value.
 * @param $ref2
 *   Hex color value.
 * @param $target
 *   Hex color value.
 */
function _fivestar_color_shift($given, $ref1, $ref2, $target) {

  // We assume that ref2 is a blend of ref1 and target and find
  // delta based on the length of the difference vectors:

  // delta = 1 - |ref2 - ref1| / |white - ref1|
  $target = _color_unpack($target, true);
  $ref1 = _color_unpack($ref1, true);
  $ref2 = _color_unpack($ref2, true);
  for ($i = 0; $i < 3; ++$i) {
    $numerator += ($ref2[$i] - $ref1[$i]) * ($ref2[$i] - $ref1[$i]);
    $denominator += ($target[$i] - $ref1[$i]) * ($target[$i] - $ref1[$i]);
  }
  $delta = ($denominator > 0) ? (1 - sqrt($numerator / $denominator)) : 0;

  // Calculate the color that ref2 would be if the assumption was true.
  for ($i = 0; $i < 3; ++$i) {
    $ref3[$i] = $target[$i] + ($ref1[$i] - $target[$i]) * $delta;
  }

  // If the assumption is not true, there is a difference between ref2 and ref3.
  // We measure this in HSL space. Notation: x' = hsl(x).
  $ref2 = _fivestar_color_rgb2hsl($ref2);
  $ref3 = _fivestar_color_rgb2hsl($ref3);
  for ($i = 0; $i < 3; ++$i) {
    $shift[$i] = $ref2[$i] - $ref3[$i];
  }

  // Take the given color, and blend it towards the target.
  $given = _color_unpack($given, true);
  for ($i = 0; $i < 3; ++$i) {
    $result[$i] = $target[$i] + ($given[$i] - $target[$i]) * $delta;
  }

  // Finally, we apply the extra shift in HSL space.
  // Note: if ref2 is a pure blend of ref1 and target, then |shift| = 0.
  $result = _fivestar_color_rgb2hsl($result);
  for ($i = 0; $i < 3; ++$i) {
    $result[$i] = min(1, max(0, $result[$i] + $shift[$i]));
  }
  $result = _fivestar_color_hsl2rgb($result);

  // Return hex color.
  return _fivestar_color_pack($result, true);
}

/**
 * Convert a hex triplet into a GD color.
 */
function _fivestar_color_gd($img, $hex) {
  $c = array_merge(array($img), _color_unpack($hex));
  return call_user_func_array('imagecolorallocate', $c);
}

/**
 * Blend two hex colors and return the resulting hex color.
 */
function _fivestar_color_blend($hex1, $hex2, $alpha) {
  $in1 = _fivestar_color_unpack($hex1);
  $in2 = _fivestar_color_unpack($hex2);
  for ($i = 0; $i < 3; ++$i) {
    $out[] = $in1[$i] + ($in2[$i] - $in1[$i]) * $alpha;
  }
  return _fivestar_color_pack($out);
}

/**
 * Convert a hex color into an RGB triplet.
 */
function _fivestar_color_unpack($hex, $normalize = false) {
  if (strlen($hex) == 4) {
    $hex = $hex[1] . $hex[1] . $hex[2] . $hex[2] . $hex[3] . $hex[3];
  }
  $c = hexdec($hex);
  for ($i = 16; $i >= 0; $i -= 8) {
    $out[] = (($c >> $i) & 0xFF) / ($normalize ? 255 : 1);
  }
  return $out;
}

/**
 * Convert an RGB triplet to a hex color.
 */
function _fivestar_color_pack($rgb, $normalize = false) {
  foreach ($rgb as $k => $v) {
    $out |= (($v * ($normalize ? 255 : 1)) << (16 - $k * 8));
  }
  return '#'. str_pad(dechex($out), 6, 0, STR_PAD_LEFT);
}

/**
 * Convert a HSL triplet into RGB
 */
function _fivestar_color_hsl2rgb($hsl) {
  $h = $hsl[0];
  $s = $hsl[1];
  $l = $hsl[2];
  $m2 = ($l <= 0.5) ? $l * ($s + 1) : $l + $s - $l*$s;
  $m1 = $l * 2 - $m2;
  return array(_fivestar_color_hue2rgb($m1, $m2, $h + 0.33333),
               _fivestar_color_hue2rgb($m1, $m2, $h),
               _fivestar_color_hue2rgb($m1, $m2, $h - 0.33333));
}

/**
 * Helper function for _color_hsl2rgb().
 */
function _fivestar_color_hue2rgb($m1, $m2, $h) {
  $h = ($h < 0) ? $h + 1 : (($h > 1) ? $h - 1 : $h);
  if ($h * 6 < 1) return $m1 + ($m2 - $m1) * $h * 6;
  if ($h * 2 < 1) return $m2;
  if ($h * 3 < 2) return $m1 + ($m2 - $m1) * (0.66666 - $h) * 6;
  return $m1;
}

/**
 * Convert an RGB triplet to HSL.
 */
function _fivestar_color_rgb2hsl($rgb) {
  $r = $rgb[0];
  $g = $rgb[1];
  $b = $rgb[2];
  $min = min($r, min($g, $b));
  $max = max($r, max($g, $b));
  $delta = $max - $min;
  $l = ($min + $max) / 2;
  $s = 0;
  if ($l > 0 && $l < 1) {
    $s = $delta / ($l < 0.5 ? (2 * $l) : (2 - 2 * $l));
  }
  $h = 0;
  if ($delta > 0) {
    if ($max == $r && $max != $g) $h += ($g - $b) / $delta;
    if ($max == $g && $max != $b) $h += (2 + ($b - $r) / $delta);
    if ($max == $b && $max != $r) $h += (4 + ($r - $g) / $delta);
    $h /= 6;
  }
  return array($h, $s, $l);
}
